package com.diagbot.facade;

import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.SearchData;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/4 14:08
 */
@Component
public class PushFacade {
    @Autowired
    private AssembleFacade assembleFacade;
    @Autowired
    private CdssCoreClient cdssCoreClient;

    /**
     * 基础推理-症状、查体、化验、辅检、诊断
     *
     * @param pushVO
     */
    public PushDTO push(PushVO pushVO) {
        SearchData searchData = new SearchData();
        BeanUtil.copyProperties(pushVO, searchData);
        //TODO 入参映射
        searchData = assembleFacade.assembleData(searchData);
        BeanUtil.copyProperties(searchData, pushVO);
        RespDTO<PushDTO> resp = cdssCoreClient.push(pushVO);
        //RespDTOUtil.respNGDealCover(resp, "远程调用推理接口失败");
        //PushDTO data = resp.data;
        PushDTO data = new PushDTO();
        List<PushBaseDTO> symptom = Lists.newArrayList();
        PushBaseDTO sympItem1 = new PushBaseDTO();
        sympItem1.setName("发热");
        symptom.add(sympItem1);
        PushBaseDTO sympItem2 = new PushBaseDTO();
        sympItem2.setName("咳嗽");
        symptom.add(sympItem2);
        data.setSymptom(symptom);
        List<PushBaseDTO> dis = Lists.newArrayList();
        PushBaseDTO disItem1 = new PushBaseDTO();
        disItem1.setName("急性胰腺炎");
        dis.add(disItem1);
        PushBaseDTO disItem2 = new PushBaseDTO();
        disItem2.setName("咳嗽");
        dis.add(disItem2);
        Map<String, List<PushBaseDTO>> disMap = new HashMap<>();
        disMap.put("可能诊断", dis);
        data.setDis(disMap);
        //TODO 出参映射
        data = assembleFacade.assemblePushDTO(data, pushVO.getHospitalId());
        return data;
    }

    /**
     * 提示信息相关推理-危急值、开单合理项、药品禁忌等
     *
     * @param indicationPushVO
     */
    public IndicationDTO indicationPush(IndicationPushVO indicationPushVO) {
        SearchData searchData = new SearchData();
        BeanUtil.copyProperties(indicationPushVO, searchData);
        //TODO 入参映射
        searchData = assembleFacade.assembleData(searchData);
        BeanUtil.copyProperties(searchData, indicationPushVO);
        IndicationDTO indicationDTO = new IndicationDTO();
        List<BillMsg> billMsgList = Lists.newArrayList();
        BillMsg item1 = new BillMsg();
        item1.setMsg("该患者年龄15,不宜开左氧氟沙星");
        item1.setOrderName("左氧氟沙星");
        item1.setOrderStandName("左氧氟沙星");
        item1.setContent("年龄15");
        item1.setType("禁忌年龄");
        billMsgList.add(item1);
        indicationDTO.setBillMsgList(billMsgList);
        List<BillMsg> criticalValList = Lists.newArrayList();
        BillMsg item2 = new BillMsg();
        item2.setMsg("该患者血常规血小板计数30,请留意");
        item2.setOrderName("");
        item2.setOrderStandName("");
        item2.setContent("血常规血小板计数30");
        item2.setType("血常规血小板计数30");
        criticalValList.add(item2);
        indicationDTO.setCriticalValList(criticalValList);
        List<BillMsg> highRiskList = Lists.newArrayList();
        BillMsg item3 = new BillMsg();
        item3.setMsg("盐酸肾上腺素注射液为高风险药品，请留意");
        item3.setOrderName("盐酸肾上腺素注射液");
        item3.setOrderStandName("盐酸肾上腺素注射液");
        item3.setContent("");
        item3.setType("药品");
        highRiskList.add(item3);
        indicationDTO.setHighRiskList(highRiskList);
        //RespDTO<IndicationDTO> resp = cdssCoreClient.indication(indicationPushVO);
        //RespDTOUtil.respNGDealCover(resp, "远程调用推理接口失败");
        //IndicationDTO indicationDTO=resp.data;
        return indicationDTO;
    }
}